/**
 * @(#)sidt.cpp 2000/10/08
 *
 * @author Ice
 * @version 1.00
 *
 * ICE Team free software group
 *
 * This file is part of TSID.
 * See README for copyright notice.
 *
 * note: some comments are in the style of Javadoc of Java language!
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

#ifndef _SIDT
#define _SIDT


#include "sidt.h"

#include <string.h>
#include <iostream.h>
#include <fstream.h>

/**
 * Construnct the SIDT
 *
 * @param absFilePath the file path name
 */
SIDT::SIDT(char *absFilePath) {
  PDEBUG("Construct the SIDT\n");

  versionString="SIDT 1.00 by Ice Team Free Software Group (C) 2000-2001\n";

  lastError=NO_SIDT_ERROR;      // no error
  sidtIndex=-1;                 // engine not initialized

  // make a phisical copy of the passed argument
  SIDT::absFilePath=new char[strlen(absFilePath)+1];
  strcpy(SIDT::absFilePath, absFilePath);

  // erase memory locations
  memset(&sidt, 0, sizeof(sidt));
}

/**
 * Destruct the SIDT
 */
SIDT::~SIDT() {
  PDEBUG("Destruct the SIDT\n");

  delete absFilePath;
}

/**
 * Add the passed information to the vector. The vector will be ordinate.
 *
 * @param song the number of song to update or add
 * @param second the number of second to add
 * @param number the number of time to add (default=1)
 */
void SIDT::add(int song, long second, long number=1) {
  // scan all the vector
  for (int i=0; i<MAX_SONGS; i++) {
    if ((sidt[i].song==song) || (sidt[i].song==0)) {
      PDEBUG("Update field for tune #" << song << endl);
      sidt[i].song=song;
      sidt[i].second+=second;
      sidt[i].number+=number;
      break;
    }
    if (song<sidt[i].song) {

      PDEBUG("Creating a free space in record\n");

      // create a space for the new song
      for (int j=MAX_SONGS-1; j>i; j--) {
        sidt[j]=sidt[j-1];
      }
      PDEBUG("Add field for tune #" << song << endl);
      sidt[i].song=song;
      sidt[i].second=second;
      sidt[i].number=number;
      break;
    }
  }
}

/**
 * Write the sidt records in the file
 *
 * @return true if operation ok
 */
bool SIDT::write() {
  PDEBUG("Write information to the file\n");

  lastError=NO_SIDT_ERROR;

  //create the file, erasing the previous if present
  ofstream file(absFilePath, ios::out);

  if (!file) {
    PDEBUG("Error creating file: " << absFilePath << endl);
    lastError=FILE_CREATE;
    return false;
  }

  // scan all the vector
  for (int i=0; i<MAX_SONGS; i++) {
    if (sidt[i].song!=0) {
      // write output text line in the file
      file << "#" << sidt[i].song << " " << sidt[i].second
           << " " << sidt[i].number << endl;
    }
  }
  PDEBUG("Write operation ended\n");
  return true;
}

/**
 * Read the record from a file if it's present. The readed record id add to
 * the actual record.
 *
 * @return true if operation is ok
 */
bool SIDT::read() {
  char ch;
  unsigned int song=0;
  unsigned int second=0;
  unsigned int number=0;

  PDEBUG("Open the File for read it\n");

  lastError=NO_SIDT_ERROR;

  // open the file
  ifstream file(absFilePath, ios::in);

  if (!file) {
    // nothing to do
    PDEBUG("Nothing to do: file not opened\n");
    return true;
  }

  while (file.get(ch)) {
    if (ch!='#') {
      cerr << "Error reading file: no # char found\n";
      lastError=FILE_FORMAT_1;    // missing #
      return false;
    }

    file >> song;
    file.get(ch);
    if (ch!=' ') {
      cerr << "Error reading file: no space found\n";
      lastError=FILE_FORMAT_2;    // missing space
      return false;
    }

    file >> second;
    file.get(ch);
    if (ch!=' ') {
      cerr << "Error reading file: no space found\n";
      lastError=FILE_FORMAT_2;    // missing space
      return false;
    }

    file >> number;
    file.get(ch);
    if (ch!='\n') {
      cerr << "Error reading file: no carriage return found\n";
      lastError=FILE_FORMAT_3;    // missing carriage return
      return false;
    }

    PDEBUG("Add the information readed from file: song #" << song << endl);
    // add the information to the record
    add(song, second, number);
  }
  return true;
}

/**
 * Init the engin for getting the next song information
 */
void SIDT::initForGet() {
  sidtIndex=0;   // go to the first element
}

/**
 * Get the next available song information from the sidt record
 *
 * @param song the returned song number
 * @param second the returned number of seconds
 * @param number the returned number of time
 * @return true if the record is present
 */
bool SIDT::getNext(unsigned int *song, unsigned long *second,
                                       unsigned long *number) {
  if (sidtIndex==-1) {
    return false;            // the engine is not initialized
  }

  if (sidt[sidtIndex].song==0) {
    return false;            // no other songs present
  }

  *song=sidt[sidtIndex].song;
  *second=sidt[sidtIndex].second;
  *number=sidt[sidtIndex].number;

  sidtIndex++;
  return true;
}

#endif // _SIDT.CPP


